//--------------------------------------------------------------
// File     : stm32_ub_i2c2.c
// Datum    : 17.07.2015
// Version  : 1.1
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F746
// IDE      : OpenSTM32
// GCC      : 4.9 2015q2
// Module   : CubeHAL
// Funktion : I2C-LoLevel-Funktionen (I2C-2)
//
// Hinweis  : mgliche Pinbelegungen
//            I2C2 : SCL : [PB10, PF1, PH4] 
//                   SDA : [PB11, PF0, PH5]
//            externe PullUp-Widerstnde an SCL+SDA notwendig
//--------------------------------------------------------------

//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_i2c2.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_HAL_I2C2_MspInit(void);



//--------------------------------------------------------------
// globale Variabeln
//-------------------------------------------------------------- 
I2C_HandleTypeDef I2c2Handle; 


//--------------------------------------------------------------
// Init von I2C2
//-------------------------------------------------------------- 
void UB_I2C2_Init(void)
{
  static uint8_t init_ok=0;

  // initialisierung darf nur einmal gemacht werden
  if(init_ok!=0) {
    return;
  } 

  I2c2Handle.Instance             = I2C2;
  I2c2Handle.Init.Timing          = I2C2_TIMING;
  I2c2Handle.Init.OwnAddress1     = 0;
  I2c2Handle.Init.AddressingMode  = I2C_ADDRESSINGMODE_7BIT;
  I2c2Handle.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  I2c2Handle.Init.OwnAddress2     = 0;
  I2c2Handle.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  I2c2Handle.Init.NoStretchMode   = I2C_NOSTRETCH_DISABLE;
  P_HAL_I2C2_MspInit();
  HAL_I2C_Init(&I2c2Handle);

  HAL_I2CEx_ConfigAnalogFilter(&I2c2Handle,I2C_ANALOGFILTER_ENABLE);

  // init Mode speichern
  init_ok=1;
}


//--------------------------------------------------------------
// Auslesen einer Adresse per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Register Adresse die gelesen wird
//
// Return_wert :
//  0...255 , Bytewert der gelesen wurde
//  < 0     , Error
//--------------------------------------------------------------
int16_t UB_I2C2_ReadByte(uint8_t slave_adr, uint8_t adr)
{
  int16_t ret_wert=0;

  I2C2_DATA[0]=adr;

  // adresse senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 1, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }  
  if(ret_wert!=0) return(ret_wert);
  
  // daten lesen
  if(HAL_I2C_Master_Receive(&I2c2Handle, (uint16_t)slave_adr, (uint8_t *)I2C2_DATA, 1, I2C2_TIMEOUT) != HAL_OK)
  {
    ret_wert=-3;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-4;
  } 
  if(ret_wert!=0) return(ret_wert);

  // daten
  ret_wert=I2C2_DATA[0];

  return(ret_wert);
}


//--------------------------------------------------------------
// Beschreiben einer Adresse per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Register Adresse die beschrieben wird
// wert      => Bytewert der geschrieben wird
//
// Return_wert :
//    0   , Ok
//  < 0   , Error
//--------------------------------------------------------------
int16_t UB_I2C2_WriteByte(uint8_t slave_adr, uint8_t adr, uint8_t wert)
{
  int16_t ret_wert=0;  

  I2C2_DATA[0]=adr;
  I2C2_DATA[1]=wert;
  
  // adresse und wert senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 2, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  } 

  return(ret_wert);
}



//--------------------------------------------------------------
// Auslesen mehrerer Adresse per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Start Register Adresse ab der gelesen wird
// cnt       => Anzahl der Bytewert die gelesen werden sollen
// Daten die gelesen worden sind, stehen danach in "I2C2_DATA"
//
// Return_wert :
//    0   , Ok
//  < 0   , Error
//--------------------------------------------------------------
int16_t UB_I2C2_ReadMultiByte(uint8_t slave_adr, uint8_t adr, uint8_t cnt)
{
  int16_t ret_wert=0;

  I2C2_DATA[0]=adr;

  // adresse senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 1, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }  
  if(ret_wert!=0) return(ret_wert);
  
  // daten lesen
  if(HAL_I2C_Master_Receive(&I2c2Handle, (uint16_t)slave_adr, (uint8_t *)I2C2_DATA, cnt, I2C2_TIMEOUT) != HAL_OK)
  {
    ret_wert=-3;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-4;
  } 
  if(ret_wert!=0) return(ret_wert);

  return(ret_wert);
}

//--------------------------------------------------------------
// Beschreiben mehrerer Adresse per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Start Register Adresse ab der beschrieben wird
// cnt       => Anzahl der Bytewert die geschrieben werden sollen
// Daten die geschrieben werden sollen, mssen in "I2C2_DATA" stehen
//
// Return_wert :
//    0   , Ok
//  < 0   , Error
//--------------------------------------------------------------
int16_t UB_I2C2_WriteMultiByte(uint8_t slave_adr, uint8_t adr, uint8_t cnt)
{
  int16_t ret_wert=0;
  uint16_t n;

  for(n=cnt;n>0;n--) {
    I2C2_DATA[n]=I2C2_DATA[n-1];    
  } 
  I2C2_DATA[0]=adr;
  
  // adresse und alle daten senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, cnt+1, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }
  
  return(ret_wert);
}


//--------------------------------------------------------------
// Schreiben eines Kommandos per I2C an einen Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// cmd       => Kommando das gesendet wird
//
// Return_wert :
//    0   , Ok
//  < 0   , Error
//--------------------------------------------------------------
int16_t UB_I2C2_WriteCMD(uint8_t slave_adr, uint8_t cmd)
{
  int16_t ret_wert=0;

  I2C2_DATA[0]=cmd;

  // command senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 1, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// Auslesen einer Adresse (16bit) per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Register Adresse die gelesen wird
//
// Return_wert :
//  0...255 , Bytewert der gelesen wurde
//  < 0     , Error
//--------------------------------------------------------------
int16_t UB_I2C2_ReadByte16(uint8_t slave_adr, uint16_t adr)
{
  int16_t ret_wert=0;
  uint8_t lo,hi;
  
  // Hi und Lo Adresse
  lo=(adr&0x00FF);
  hi=(adr>>8);  

  I2C2_DATA[0]=hi;
  I2C2_DATA[1]=lo;

  // adresse senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 2, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }  
  if(ret_wert!=0) return(ret_wert);
  
  // daten lesen
  if(HAL_I2C_Master_Receive(&I2c2Handle, (uint16_t)slave_adr, (uint8_t *)I2C2_DATA, 1, I2C2_TIMEOUT) != HAL_OK)
  {
    ret_wert=-3;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-4;
  } 
  if(ret_wert!=0) return(ret_wert);

  // daten
  ret_wert=I2C2_DATA[0];

  return(ret_wert);

}

//--------------------------------------------------------------
// Beschreiben einer Adresse (16bit) per I2C von einem Slave
// slave_adr => I2C-Basis-Adresse vom Slave
// adr       => Register Adresse die beschrieben wird
// wert      => Bytewert der geschrieben wird
//
// Return_wert :
//    0   , Ok
//  < 0   , Error
//--------------------------------------------------------------
int16_t UB_I2C2_WriteByte16(uint8_t slave_adr, uint16_t adr, uint8_t wert)
{
  int16_t ret_wert=0;
  uint8_t lo,hi;
  
  // Hi und Lo Adresse
  lo=(adr&0x00FF);
  hi=(adr>>8);  

  I2C2_DATA[0]=hi;
  I2C2_DATA[1]=lo;
  I2C2_DATA[2]=wert;

  // adresse und wert senden
  if(HAL_I2C_Master_Transmit(&I2c2Handle, (uint16_t)slave_adr, (uint8_t*)I2C2_DATA, 3, I2C2_TIMEOUT)!= HAL_OK)
  {
    ret_wert=-1;
    if (HAL_I2C_GetError(&I2c2Handle) != HAL_I2C_ERROR_AF) ret_wert=-2;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// kleine Pause (ohne Timer)
//--------------------------------------------------------------
void UB_I2C2_Delay(volatile uint32_t nCount)
{
  while(nCount--)
  {
  }
}


//--------------------------------------------------------------
// interne Funktion
//-------------------------------------------------------------- 
void P_HAL_I2C2_MspInit(void)
{
  GPIO_InitTypeDef  GPIO_InitStruct;
  RCC_PeriphCLKInitTypeDef  RCC_PeriphCLKInitStruct;
  
  // I2C-Clock
  RCC_PeriphCLKInitStruct.PeriphClockSelection = RCC_PERIPHCLK_I2C2;
  RCC_PeriphCLKInitStruct.I2c2ClockSelection = RCC_I2C2CLKSOURCE_PCLK1 ;
  HAL_RCCEx_PeriphCLKConfig(&RCC_PeriphCLKInitStruct);

  // GPIO-Clock
  UB_System_ClockEnable(I2C2_SCL_PORT);
  UB_System_ClockEnable(I2C2_SDA_PORT);
  // enable
  __HAL_RCC_I2C2_CLK_ENABLE(); 

  // GPIO
  GPIO_InitStruct.Pin       = I2C2_SCL_PIN;
  GPIO_InitStruct.Mode      = GPIO_MODE_AF_OD;
  GPIO_InitStruct.Pull      = GPIO_PULLUP;
  GPIO_InitStruct.Speed     = GPIO_SPEED_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
  HAL_GPIO_Init(I2C2_SCL_PORT, &GPIO_InitStruct);
  
  GPIO_InitStruct.Pin       = I2C2_SDA_PIN;
  GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
  HAL_GPIO_Init(I2C2_SDA_PORT, &GPIO_InitStruct);
} 
